function ACLUtilities(restEngine, templateManager, ZHPromise, ScreenService, ZhPreferences,
                      ZhFilterPreferences, General, requestDataSourceEngine, AclMessages, MessageService)
{
    var self = this;

    this.callRestEngine = function (dataSource, requestType, requestValue) {
        var defer = ZHPromise.defer();
        var params = {};

        switch (requestType) {
            case "Row":
                params = {
                    "requestType": "Row",
                    "row": requestValue
                };
                break;
            case "DataSet":
                params = {
                    "requestType": "DataSet",
                    "dataset": requestValue
                };
                break;
            case "FilterData":
                params = {
                    "requestType": "FilterData",
                    "filter": requestValue
                };
                break;
            default:
                return defer.reject("Erro na passagem de parâmetros para a requisição. O valor para requestType deve ser 'Row', 'DataSet' ou 'FilterData'.");
        }

        return restEngine.request(dataSource, params);
    };

    this.changeWidgetAcessibility = function (widgets, enabled) {
        widgets.forEach(function (widget) {
            widget.enabled = enabled;
        });

        templateManager.updateTemplate();
    };

    this.formWidgetSwitchTemplate = function (widget) {
        if ((widget.template != "widget/form.html") && (widget.template != "widget/form-view.html")) return false;

        if (!widget.templateToForm) widget.templateToForm = "widget/form.html";
        if (!widget.templateToView) widget.templateToView = "widget/form-view.html";

        if (widget.template == widget.templateToForm) widget.template = widget.templateToView;
        else if (widget.template == widget.templateToView) widget.template = widget.templateToForm;

        templateManager.updateTemplate();

        return true;
    };

    this.widgetSave = function (widget) {
        var deferred = ZHPromise.defer();

        if (widget.isValid()) {
            widget.currentRow.changed = true;

            if (!widget.currentRow.__is_new)
                widget.currentRow.__is_new = false;
            else if (widget.paramsNewCode && widget.paramsNewCode.__is_new_code) {
                widget.currentRow.__using_unit = widget.paramsNewCode.__using_unit || false;
                widget.currentRow.__new_code_pk = widget.paramsNewCode.__new_code_pk;
                widget.currentRow.__table_pk = widget.paramsNewCode.__table_pk;
                widget.currentRow.__new_code_length = widget.paramsNewCode.__new_code_length || 5;
                widget.currentRow.__cast_number = widget.paramsNewCode.__cast_number || false;
            }

            widget.saveRow().then(function (response) {
                widget.currentRow.changed = false;
                delete widget.backupRow;

                if (self.formWidgetSwitchTemplate(widget) && widget.parent) {
                    widget.view();

                    var widgets = widget.parent.widgets;
                    self.changeWidgetAcessibility(widgets.concat(widget.parent), true);
                } else {
                    if (widget.currentRow.__is_new) {
                        ScreenService.closeSwipe();
                        widget.list();
                    } else widget.view();

                    self.changeWidgetAcessibility(widget.widgets, true);
                }

                if (widget.currentRow.__is_new) widget.currentRow.__is_new = false;
                else widget.reload();

                deferred.resolve(response);
            }.bind(this), function (error) {
                var errorMessage = 'Timeout';

                if (error.data) errorMessage = error.data.error;

                ScreenService.showMessage(errorMessage);
                deferred.reject(errorMessage);
            });
        } else deferred.reject(ScreenService.i18n("Preencha todos os campos corretamente!"));

        return deferred;
    };

    this.widgetEdit = function (widget) {
        if (!widget.editing) {
            //Trata a edição de uma widget filha do tipo form
            if (self.formWidgetSwitchTemplate(widget) && widget.parent) {
                var widgetsToDeactivate = widget.parent.widgets;

                self.changeWidgetAcessibility(
                    widgetsToDeactivate.filter(function (subwidget) {
                        return subwidget.name != widget.name;
                    }).concat(widget.parent), //retorna um array com a widget parent e suas widgets filhas, exceto aquela que será editada
                    false);
            } else //trata a edição dos demais casos
                self.changeWidgetAcessibility(widget.widgets, false);

            //propriedades a serem ignoradas na criação do backup
            var ignoredKeys = ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders",
                               "__index", "__isSelected", "__notSelectable", "$$hashKey"];

            //faz backup da currentRow para permitir a recuperação dos dados
            widget.backupRow = Util.clone(widget.currentRow, ignoredKeys);
            widget.edit();
            return true;
        } else return false;
    };
    
    /*
	 * Procedimento que cancela o formulário de uma
	 * widget, seja este de inserção ou de edição, e
	 * desbloqueia as demais widgets.
	 *
	 * @param object widget
	 */
	this.widgetCancelForm = function (widget) {
		//trata o cancelamento de uma widget filha do tipo form
		if (this.formWidgetSwitchTemplate(widget) && widget.parent) {
			var widgets = widget.parent.widgets;

			this.changeWidgetAcessibility(widgets.concat(widget.parent), true);
		} else //trata o cancelamento dos demais casos
			this.changeWidgetAcessibility(widget.widgets, true);

		//caso a widget estiver em modo de inserção, remove a row e fecha o swipe
		if (widget.currentRow.__is_new) {
			var indexOfCurrentRow = widget.dataSource.data.indexOf(widget.currentRow);
			if (indexOfCurrentRow !== -1) {
				widget.dataSource.data.pop();
			}

			widget.list();

			ScreenService.closeSwipe();
		} else { //caso a widget estiver em modo de edição, recupera o backup e entra no modo de visualização
			Object.keys(widget.backupRow).forEach(function (key) {
				widget.currentRow[key] = widget.backupRow[key];
			}); //copia dados coluna por coluna para não perder referência do objeto da currentRow
			widget.view();
		}
	};
	
	this.startFilter = function(widget, currentRow){
        ZhPreferences.getDefaultWidgetPreferences(widget.id).then(function(result) {
            if (result) {
                ZhFilterPreferences.openFilterWidget(widget, currentRow);
            }
        });
    };

    this.applyDefaultFilterOnEnter = function(widget) {
        var time = 500;
        return setTimeout(function() {
            self.setDefaultFilterOnEnter(widget);
        }, time);
    };

    this.setDefaultFilterOnEnter = function(widget){
        if (widget.inListMode()) {
            if (self.isAppliedFilter(widget)) {
                widget.reload();
            } else if (widget.currentRowDefaultFilter) {
                self.startFilter(widget, widget.currentRowDefaultFilter);
            } else {
                self.startFilter(widget, widget.currentRow);
            }
        }
    };
    
    this.isAppliedFilter = function(widget){
        return !!widget.aclAppliedFilter;
    };

    this.setFilterInit = function (widget) {
        widget.aclAppliedFilter = true;
    };

    this.initializeFieldSelectAutocomplete = function(field){
        field.initializeField();
    };
    
    this.genericCancelEvent = function(widget, primaryProprety) {
        primaryProprety = primaryProprety ? primaryProprety :"ID";
        var primaryValue = widget.currentRow[primaryProprety];
        if (widget.currentRow.__is_new) {
            ScreenService.closeSwipe();
            widget.reload();
        } else {
            widget.reload().then(function(){
                widget.view();
                return widget.dataSource.data.some(function (row) {
                    if (row[primaryProprety] == primaryValue) {
                        widget.currentRow = row;
                        return true;
                    }
                });
            });
        }
        templateManager.updateTemplate();
    };
    
    this.validateCPF = function (cpf) {
        if(cpf && !General.validateCPF(cpf)){
            ScreenService.showMessage(ScreenService.i18n("Número de CPF inválido."));
            return false;
        }
        return true;
    };
    
    this.checkRequirements = function(widget) {
        if(!widget.isValid()) {
            ScreenService.showMessage(ScreenService.i18n('Preencha todos os campos marcados com asterisco!'));
            return false;
        }
        return true;
    };
    
    this.getRowInDataSource = function (dataSource, primaryProperty, primaryValue) {
        var currentRow = null;
        dataSource.data.some(function (row) {
            if (row[primaryProperty] == primaryValue) {
                currentRow = row;
                return true;
            }
        });
        return currentRow;
    };
    
    this.setDataSourceAfterSave = function(widget, filter, valueIndex){
        var widgetDataSource = widget.dataSource.data;
        return requestDataSourceEngine.getDataSource(widget.dataSource.name, filter).then(function (response) {
            var dataset = response.dataset[widget.dataSource.name.split('/')[1]];
            var isRow = widgetDataSource.some(function(data, index){
                if(data[valueIndex] == dataset[0][valueIndex]){
                    widgetDataSource[index] = dataset[0];
                    return true;
                }
            });
            if(!isRow) {
                widgetDataSource.push(dataset[0]);
            }
            return dataset[0];
        });
    };
    
    this.genericDeleteEvent = function (widget){
        return widget.dataSource.remove([widget.currentRow]).then(function(data){
            return ScreenService.closeSwipe().then(function(){
                return widget.reload();
            });
        });
    };
    
    this.getOnlyNumberInString = function(string){
        if (string) return string.split(/\D+/).join("");
    };

    this.validateNewPassword = function (newPassword, confirmPassword, password) {

        if (newPassword != confirmPassword) {
            MessageService.showMessageByCode('ALE_PASSWORD_NOT_MATCH', "ALERT");
            return false;
        }

        if (password && password === newPassword){
            MessageService.showMessageByCode('ALE_NEW_PASSWORD_EQUAL_OLD', "ALERT");
            return false;
        }

        return true;
    };
    
    this.getTypeAccessProfile = function(){
        return requestDataSourceEngine.getDataSource('acl#/findTypeAccessProfile');
    };
	
    this.isJson = function(str) {
        var result = null;
        try {
            result = JSON.parse(str);
        } catch (e) { }
        return result;
    };

    this.showErrorMessage = function(response) {
        return MessageService.getMessageByCode('lib#UNEXPECTED_ERROR').then(function(defaultMessage) {
            if (response && response.data && response.data.error) {
                var error = response.data.error;
                if (self.isJson(error)) {
                    error = JSON.parse(error);
                    var params = error.PARAMS || null;
                    var messageCode = error.CODE || 'lib#UNEXPECTED_ERROR';
                    if(messageCode == 'INVALID_PASSWORD_MASK') {
                        return showInvalidPasswordMaskMessage(messageCode, params);
                    } else {
                        return MessageService.getMessageByCode('acl#' + messageCode).then(function(message) {
                            return ScreenService.showMessage(message, 'ERROR', null, params);
                        }).catch(function() {
                            return ScreenService.showMessage(messageCode, 'ERROR', null, params);
                        });
                    }
                } else {
                    var message = Util.isEmptyOrBlank(error) ? defaultMessage : error.split("Exception:").pop();
                    return ScreenService.showMessage(message, 'ERROR');
                }
            } else {
                return ScreenService.showMessage(defaultMessage, 'ERROR');
            }
        });
    };

    var showInvalidPasswordMaskMessage = function(messageCode, params) {
        var promises = [];
        var messages = '';
        params.forEach(function(param) {
            if(param && param.EXISTS_MASK && param.INVALID_MASK) {
                var messagePromise = MessageService.getMessageByCode('acl#' + param.MESSAGE).then(function(message) {
                    messages += '<li align="left">' + ScreenService.i18n(message, param.PARAMS) + '</li>';
                });
                promises.push(messagePromise);
            }
        });
        ZHPromise.all(promises).then(function() {
            MessageService.showMessageByCode('acl#' + messageCode, { validations: messages }, { type: 'ALERT' });
        });
    };

}

Configuration(function(ContextRegister) {
    ContextRegister.register('ACLUtilities', ACLUtilities);
});
